// 全局变量存储纠音字典
let pronounceDict = {};

function showCharacterSelector(type) {
    const selector = document.getElementById('characterSelector');
    selector.dataset.type = type;
    selector.style.display = 'block';
}

function selectCharacter(name) {
    const type = document.getElementById('characterSelector').dataset.type;
    if(type === 'default') {
        document.getElementById('defaultCharacterBtn').textContent = name;
        document.getElementById('defaultCharacterBtnVal').value = name;
    } else {
        insertCharacter(name);
    }
    document.getElementById('characterSelector').style.display = 'none';
}

function showPronounceDialog() {
    document.getElementById('pronounceDialog').style.display = 'block';
}

function savePronounce() {
    const word = document.getElementById('wordInput').value;
    const pinyin = document.getElementById('pinyinInput').value.split(',');

    if(word && pinyin) {
        pronounceDict[word] = pinyin;
        updatePronounceList();
        // clearPronounceInputs();
    }
}

function updatePronounceList() {
    const list = document.getElementById('pronounceList');
    list.innerHTML = '';

    for(let [word, pinyin] of Object.entries(pronounceDict)) {
        const item = document.createElement('div');
        item.className = 'pronounce-item';
        item.innerHTML = `
            <span>${word} -> ${pinyin.join(',')}</span>
            <button onclick="deletePronounce('${word}')">删除</button>
        `;
        list.appendChild(item);
    }
}

function deletePronounce(word) {
    // 检查是否存在该单词
    if (pronounceDict[word]) {
        delete pronounceDict[word]; // 从字典中删除该单词
        updatePronounceList(); // 更新纠音列表
    }
}

function switchTab(tab) {
    document.querySelectorAll('.tab-button').forEach(btn => btn.classList.remove('active'));
    document.querySelectorAll('.card').forEach(card => card.style.display = 'none');

    document.querySelector(`button[onclick="switchTab('${tab}')"]`).classList.add('active');
    document.getElementById(`${tab}-tab`).style.display = 'block';
}

let fileQueue = [];
let processing = false;
let batchStartBtn = document.createElement('button');
batchStartBtn.className = 'main-button';
batchStartBtn.textContent = '开始批量合成语音';
batchStartBtn.onclick = processQueue;
document.getElementById('batch-tab').appendChild(batchStartBtn);

document.getElementById('txtFiles').addEventListener('change', async function(e) {
    const files = Array.from(e.target.files);
    const limitLines = document.getElementById('limitLines').checked;

    for(let file of files) {
        const fileItem = createFileListItem(file.name);
        document.getElementById('fileList').appendChild(fileItem);

        try {
            // 首先尝试使用 UTF-8 读取
            const contentUTF8 = await readFileAsText(file, 'utf-8');
            // 检查是否有乱码(简单判断)
            if (hasGarbledText(contentUTF8)) {
                // 如果检测到乱码,尝试用 GBK 重新读取
                const content = await readFileAsText(file, 'gbk');
                processFileContent(content, limitLines, fileItem, file.name);
            } else {
                processFileContent(contentUTF8, limitLines, fileItem, file.name);
            }
        } catch (error) {
            console.error('读取文件失败:', error);
            fileItem.querySelector('.status').textContent = '读取失败';
        }
    }
});



function updateFileSelection(checkbox, fileName) {
  const file = fileQueue.find(f => f.name === fileName);
  if(file) {
    file.selected = checkbox.checked;
  }
}

function createFileListItem(fileName) {
    const div = document.createElement('div');
    div.className = 'file-item';
    div.innerHTML = `
        <input type="checkbox" class="file-checkbox" checked>
        <div class="file-name">${fileName}</div>
        <div class="file-status">等待处理</div>
        <div class="file-audio"></div>
        <div class="button-group">
            <button class="down-btn" style="display: none">下载音频</button>
            <button class="delete-btn" onclick="removeFile(this)">删除</button>
        </div>
    `;
    div.querySelector('.file-checkbox').addEventListener('click', function(e) {
      updateFileSelection(this, fileName);
    });
    return div;
}

// 添加全选/取消全选按钮
document.getElementById('batch-tab').insertAdjacentHTML('beforeend', `
    <div class="checkbox-controls">
        <button class="styled-button" onclick="selectAllFiles()">全选</button>
        <button class="styled-button" onclick="unselectAllFiles()">取消全选</button>
        <button class="styled-button" onclick="batchDownloadAudios()">批量下载选中音频</button>
    </div>
`);

// 全选函数
function selectAllFiles() {
    document.querySelectorAll('.file-checkbox').forEach(checkbox => {
        checkbox.checked = true;
    });
}

// 取消全选函数
function unselectAllFiles() {
    document.querySelectorAll('.file-checkbox').forEach(checkbox => {
        checkbox.checked = false;
    });
}

async function batchDownloadAudios() {
    const selectedFiles = document.querySelectorAll('.file-checkbox:checked');
    if(selectedFiles.length === 0) {
        alert('请至少选择一个文件进行下载');
        return;
    }

    let completedCount = 0;
    let skippedCount = 0;

    // 创建下载队列
    for(let checkbox of selectedFiles) {
        const fileItem = checkbox.closest('.file-item');
        const statusElement = fileItem.querySelector('.file-status');
        const audioElement = fileItem.querySelector('audio');
        const fileName = fileItem.querySelector('.file-name').textContent;

        // 检查文件状态是否为"已完成"且存在音频元素
        if(statusElement.textContent === '已完成' && audioElement && audioElement.src) {
            try {
                const link = document.createElement('a');
                // 使用原始音频URL加上文件名
                const audioUrl = new URL(audioElement.src);
                link.href = `${audioUrl.pathname}/${fileName}`;
                link.download = fileName + '.mp3';

                // 触发下载
                document.body.appendChild(link);
                link.click();
                document.body.removeChild(link);

                completedCount++;
                // 添加小延迟，避免浏览器同时触发太多下载
                await new Promise(resolve => setTimeout(resolve, 500));
            } catch(error) {
                console.error(`下载文件 ${fileName} 失败:`, error);
            }
        } else {
            skippedCount++;
        }
    }

    // 显示处理结果
    if(completedCount > 0 || skippedCount > 0) {
        let message = '';
        if(completedCount > 0) {
            message += `成功下载 ${completedCount} 个文件。`;
        }
        if(skippedCount > 0) {
            message += `\n跳过 ${skippedCount} 个未完成的文件。`;
        }
        alert(message);
    }
}

// 添加删除文件的函数
function removeFile(btn) {
    // 使用 closest() 方法找到最近的 file-item 祖先元素
    const fileItem = btn.closest('.file-item');
    if (!fileItem) {
        console.error('Cannot find file item');
        return;
    }

    const fileName = fileItem.querySelector('.file-name').textContent;

    // 从队列中移除
    fileQueue = fileQueue.filter(item => item.name !== fileName);

    // 移除DOM元素
    fileItem.remove();
}


async function processQueue() {
    const selectedFiles = document.querySelectorAll('.file-checkbox:checked');
    if(selectedFiles.length === 0) {
        alert('请至少选择一个文件进行处理');
        processing = false;
        return;
    }
    processing = true;

    for(let checkbox of selectedFiles) {
        const fileItem = checkbox.closest('.file-item');
        const fileName = fileItem.querySelector('.file-name').textContent;
        const statusElement = fileItem.querySelector('.file-status');
        const audioContainer = fileItem.querySelector('.file-audio');
        const downContainer = fileItem.querySelector('.down-btn');

        // 获取对应的文件内容
        const fileData = fileQueue.find(item => item.name === fileName);
        if(!fileData) continue;

        statusElement.textContent = '处理中...';

        // 清除之前的音频元素
        audioContainer.innerHTML = '';
        downContainer.style.display = 'none';

        const seedInput = document.getElementById('seed');
        const keepRandom = document.getElementById('keepRandom');
        let seedValue;
        if(keepRandom.checked) {
            seedValue = getRandomSeed();
            seedInput.value = seedValue;
        } else {
            if(seedInput.value === '' || seedInput.value === '-1') {
                seedValue = getRandomSeed();
                seedInput.value = seedValue;
            } else {
                seedValue = parseInt(seedInput.value);
            }
        }

        try {
            const response = await fetch('/synthesize', {
                method: 'POST',
                headers: {'Content-Type': 'application/json'},
                body: JSON.stringify({
                    character: document.getElementById('defaultCharacterBatch').value,
                    text: fileData.content,
                    speed: document.getElementById('speed').value,
                    volume: document.getElementById('volume').value,
                    fragment_interval: document.getElementById('fragment_interval').value,
                    seed: seedValue,
                })
            });

            const data = await response.json();
            if(data.status === 'success') {
                statusElement.textContent = '已完成';
                audioContainer.innerHTML = `
                    <audio controls src="${data.audio_url}"></audio>
                `;
                audioContainer.style.display = 'block';

                downContainer.style.display = 'block';
                downContainer.onclick = function() {
                    const link = document.createElement('a');
                    // link.href = data.audio_url;
                    link.href = `${data.audio_url}/${fileName}`;
                    link.download = fileName + ".mp3";
                    link.click();
                };
            } else {
                statusElement.textContent = '处理失败';
            }
        } catch(error) {
            statusElement.textContent = '处理失败';
            console.error(error);
        }
    }

    processing = false;
}



// 视频切换功能
function switchVideo(type) {
    const originalVideo = document.getElementById('originalVideo');
    const processedVideo = document.getElementById('processedVideo');

    if (type === 'original') {
        originalVideo.style.display = 'block';
        processedVideo.style.display = 'none';
    } else {
        originalVideo.style.display = 'none';
        processedVideo.style.display = 'block';
    }
}

// 处理视频
async function processVideo() {
    const video = document.getElementById('originalVideo');
    const character = document.getElementById('videoCharacter').value;
    const volume = document.getElementById('videoVolume').value;

    // 这里添加视频处理的具体逻辑
    try {
        // 示例：调用后端API
        const formData = new FormData();
        formData.append('video', video.src);
        formData.append('character', character);
        formData.append('volume', volume);

        const response = await fetch('/api/process-video', {
            method: 'POST',
            body: formData
        });

        if (response.ok) {
            const result = await response.blob();
            const processedVideo = document.getElementById('processedVideo');
            processedVideo.src = URL.createObjectURL(result);
            switchVideo('processed');
        }
    } catch (error) {
        console.error('视频处理失败:', error);
    }
}

// 下载处理后的视频
function downloadProcessedVideo() {
    const processedVideo = document.getElementById('processedVideo');
    if (processedVideo.src) {
        const link = document.createElement('a');
        link.href = processedVideo.src;
        link.download = 'processed_video.mp4';
        link.click();
    }
}
