// 使用指定编码读取文件
function readFileAsText(file, encoding) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onload = (e) => resolve(e.target.result);
        reader.onerror = (e) => reject(e);
        reader.readAsText(file, encoding);
    });
}

function cleanTtsText(text) {
    // 匹配除了中文、日文、韩文、英文数字、基本标点符号以外的字符
    const pattern = /[^\u4e00-\u9fff\u3040-\u309f\u30a0-\u30ff\uac00-\ud7af\u1100-\u11ff\u0020-\u007f，。！？；：""''（）、\n\r]/g;

    let cleanedText = text.replace(pattern, '');
    // 将多个空格替换为单个空格,但保留换行符
    cleanedText = cleanedText.replace(/[^\S\n\r]+/g, ' ');
    cleanedText = cleanedText.trim();
    return cleanedText;
}

// 简单检查是否包含乱码
function hasGarbledText(text) {
    // 检查是否包含特殊字符(可能是乱码)
    const garbledPattern = /[\u0000-\u0008\u00B7\u00FD-\u00FF\uFFFD]/g;
    return garbledPattern.test(text) || containsHighPercentageOfGarbledChars(text);
}

// 检查乱码字符的比例
function containsHighPercentageOfGarbledChars(text) {
    const totalChars = text.length;
    if (totalChars === 0) return false;

    let garbledCount = 0;
    for (let i = 0; i < totalChars; i++) {
        const charCode = text.charCodeAt(i);
        if ((charCode >= 0xD800 && charCode <= 0xDFFF) || charCode === 0xFFFD) {
            garbledCount++;
        }
    }

    return (garbledCount / totalChars) > 0.1; // 如果乱码字符超过10%则认为是乱码
}

// 处理文件内容
function processFileContent(content, limitLines, fileItem, fileName) {
    if(limitLines) {
        content = content.split('\n').slice(0, 10000).join('\n');
    }

    fileQueue.push({
        content: content,
        element: fileItem,
        name: fileName,
        selected: true
    });
}

function insertCharacter(char) {
    const textarea = document.getElementById('text');
    const selectedText = textarea.value.substring(textarea.selectionStart, textarea.selectionEnd);
    const pos = textarea.selectionStart;

    const insertText = selectedText ?
        `\n{${char}@@${selectedText}}\n` :
        `\n{${char}@@大家好，我是${char}}\n`;

    textarea.value = textarea.value.slice(0, pos) + insertText + textarea.value.slice(textarea.selectionEnd);
    const newPos = pos + insertText.length;
    textarea.setSelectionRange(newPos, newPos);
    textarea.focus();
}

function insertPause() {
    const textarea = document.getElementById('text');
    const pos = textarea.selectionStart;
    const insertText = `[p1.0]`;
    textarea.value = textarea.value.slice(0, pos) + insertText + textarea.value.slice(textarea.selectionEnd);
    const newPos = pos + insertText.length;
    textarea.setSelectionRange(newPos, newPos);
    textarea.focus();
}
async function synthesize() {
    const progress = document.getElementById('progress');
    const audioSection = document.getElementById('audioSection');
    const errorMessage = document.getElementById('errorMessage');
    const synthesizeButton = document.querySelector('.main-button');

    errorMessage.style.display = 'none';
    progress.style.display = 'block';
    audioSection.style.display = 'none';
    synthesizeButton.disabled = true;
    updateProgress();

    try {
        // const character = document.getElementById('defaultCharacterBtn').textContent;
        const character = document.getElementById('defaultCharacterBtnVal').value
        console.log(character)
        const text = document.getElementById('text').value.trim();
        if (!character) throw new Error('请选择角色');
        if (!text) throw new Error('请输入要合成的文本');

        const seedInput = document.getElementById('seed');
        const keepRandom = document.getElementById('keepRandom');
        const GenerateSrt = document.getElementById('GenerateSrt');
        let seedValue;
        if(keepRandom.checked) {
            // 如果勾选了保持随机，每次都生成新的随机值
            seedValue = getRandomSeed();
            seedInput.value = seedValue;
        } else {
            // 如果没有勾选保持随机，使用文本框的值
            if(seedInput.value === '' || seedInput.value === '-1') {
                seedValue = getRandomSeed();
                seedInput.value = seedValue;
            } else {
                seedValue = parseInt(seedInput.value);
            }
        }

        if(!GenerateSrt.checked){
            document.getElementById('src-btn').href = "javascript:void(0);";
            document.getElementById('src-btn').style.display = "none";
        }

        console.log('获取html值得的seed'+seedInput.value)

        const response = await fetch('/synthesize', {
            method: 'POST',
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                character: character,
                text: cleanTtsText(text),
                speed: document.getElementById('speed').value,
                volume: document.getElementById('volume').value,
                fragment_interval: document.getElementById('fragment_interval').value,
                seed: seedValue,
                GenerateSrt:GenerateSrt.checked,
                py_dict2: pronounceDict  // 添加纠音字典
            })
        });

        if (!response.ok) {
            const errorData = await response.json().catch(() => ({}));
            throw new Error(errorData.message || `服务器错误 (${response.status})`);
        }

        const data = await response.json();
        if (data.status === 'success') {
            const audio = document.getElementById('audio');
            const srt_link = document.getElementById('src-btn');
            audio.src = data.audio_url;
            audioSection.style.display = 'flex';
            if(data.srt_file !=""){
                srt_link.style.display = 'flex';
                srt_link.href = data.srt_file
            }
            stopProgress(); // 成功时停止跳动并填满
        } else {
            throw new Error(data.message || '合成失败');
        }

    }  catch (error) {
        errorMessage.textContent = error.message || '合成过程出现错误，请重试';
        errorMessage.style.display = 'block';
        audioSection.style.display = 'none';
        stopProgress(); // 失败时也停止跳动
    } finally {
        progress.style.display = 'none';
        synthesizeButton.disabled = false;
    }
}